//
//  AppDelegate.swift
//  WorkNow
//
//  Created by Javier Arguello on 21/4/17.
//  Copyright © 2017 uoc. All rights reserved.
//

import UIKit
import Firebase
import UserNotifications
import FirebaseInstanceID
import FirebaseMessaging
import MapKit

@UIApplicationMain
class AppDelegate: UIResponder, UIApplicationDelegate, UNUserNotificationCenterDelegate, FIRMessagingDelegate {

    var window: UIWindow?
    let gcmMessageIDKey = "gcm.message_id"
    var registrationOptions = [String: AnyObject]()

    func application(_ application: UIApplication, didFinishLaunchingWithOptions launchOptions: [UIApplicationLaunchOptionsKey: Any]?) -> Bool {
        // Override point for customization after application launch.
        
        if #available(iOS 10.0, *) {
            // For iOS 10 display notification (sent via APNS)
            UNUserNotificationCenter.current().delegate = self
            let authOptions: UNAuthorizationOptions = [.alert, .badge, .sound]
            UNUserNotificationCenter.current().requestAuthorization(
                options: authOptions,
                completionHandler: {_, _ in })
            // For iOS 10 data message (sent via FCM
            FIRMessaging.messaging().remoteMessageDelegate = self
        } else {
            let settings: UIUserNotificationSettings =
                UIUserNotificationSettings(types: [.alert, .badge, .sound], categories: nil)
            application.registerUserNotificationSettings(settings)
        }
        application.registerForRemoteNotifications()
        registerForPushNotifications()
        NotificationCenter.default.addObserver(self,
                                                         selector: #selector(tokenRefreshNotification(_:)),
                                                         name: NSNotification.Name.firInstanceIDTokenRefresh,
                                                         object: nil)
        FIRApp.configure()
        return true
    }
    
    func registerForPushNotifications() {
        UNUserNotificationCenter.current().requestAuthorization(options: [.alert, .sound, .badge]) {
            (granted, error) in
            print("Permission granted: \(granted)")
        }
    }
    
    public func application(application: UIApplication, didRegisterForRemoteNotificationsWithDeviceToken deviceToken: NSData) {
        
        FIRInstanceID.instanceID().setAPNSToken(deviceToken as Data, type: FIRInstanceIDAPNSTokenType.sandbox)
    }
    
    func tokenRefreshNotification(notification: NSNotification) {
        // NOTE: It can be nil here
        let refreshedToken = FIRInstanceID.instanceID().token()
        print("InstanceID token: \(refreshedToken)")
        Global.deviceToken = refreshedToken!
        
        connectToFcm()
    }
    
    func connectToFcm() {
        FIRMessaging.messaging().connect { (error) in
            if (error != nil) {
                print("Unable to connect with FCM. \(error)")
            } else {
                print("Connected to FCM.")
            }
        }
    }
    func tokenRefreshNotification(_ notification: Notification) {
        if let refreshedToken = FIRInstanceID.instanceID().token() {
            print("InstanceID token: \(refreshedToken)")
            Global.deviceToken = refreshedToken
        }
        
        // Connect to FCM since connection may have failed when attempted before having a token.
        //connectToFcm()
    }
    /*
    func application(application: UIApplication, didRegisterForRemoteNotificationsWithDeviceToken deviceToken: NSData)
    {
        if let refreshedToken = FIRInstanceID.instanceID().token() {
            print("InstanceID token: \(refreshedToken)")
        }
        FIRInstanceID.instanceID().setAPNSToken(deviceToken as Data, type: FIRInstanceIDAPNSTokenType.sandbox)
        FIRInstanceID.instanceID().setAPNSToken(deviceToken as Data, type: FIRInstanceIDAPNSTokenType.prod)
    }*/

    func applicationWillResignActive(_ application: UIApplication) {
        // Sent when the application is about to move from active to inactive state. This can occur for certain types of temporary interruptions (such as an incoming phone call or SMS message) or when the user quits the application and it begins the transition to the background state.
        // Use this method to pause ongoing tasks, disable timers, and invalidate graphics rendering callbacks. Games should use this method to pause the game.
    }

    func applicationDidEnterBackground(_ application: UIApplication) {
        // Use this method to release shared resources, save user data, invalidate timers, and store enough application state information to restore your application to its current state in case it is terminated later.
        // If your application supports background execution, this method is called instead of applicationWillTerminate: when the user quits.
        FIRMessaging.messaging().disconnect()
        print("Disconnected from FCM.")
    }

    func applicationWillEnterForeground(_ application: UIApplication) {
        // Called as part of the transition from the background to the active state; here you can undo many of the changes made on entering the background.
    }

    func applicationDidBecomeActive(_ application: UIApplication) {
        // Restart any tasks that were paused (or not yet started) while the application was inactive. If the application was previously in the background, optionally refresh the user interface.
        FIRMessaging.messaging().connect { error in
            if (error != nil) {
                print("Unable to connect with FCM. \(error)")
            } else {
                print("Connected to FCM.")
                if let refreshedToken = FIRInstanceID.instanceID().token() {
                    print("InstanceID token: \(refreshedToken)")
                    Global.deviceToken = refreshedToken
                }
            }
        }
    }

    func applicationWillTerminate(_ application: UIApplication) {
        // Called when the application is about to terminate. Save data if appropriate. See also applicationDidEnterBackground:.
    }
    func application(application: UIApplication, didReceiveRemoteNotification userInfo: [NSObject : AnyObject],
                     fetchCompletionHandler completionHandler: (UIBackgroundFetchResult) -> Void) {
        // If you are receiving a notification message while your app is in the background,
        // this callback will not be fired till the user taps on the notification launching the application.
        // TODO: Handle data of notification
        
        // Print message ID.
    //    if let messageID = userInfo[gcmMessageIDKey] {
      //      print("Message ID: \(messageID)")
       // }
        
        // Print full message.
        FIRMessaging.messaging().appDidReceiveMessage(userInfo)
        print("%@", userInfo)
    }
    
    func applicationReceivedRemoteMessage(_ remoteMessage: FIRMessagingRemoteMessage) {
        print(remoteMessage.appData)
        let dict = remoteMessage.appData["notification"] as! NSDictionary
        let title = dict["title"] as! String
        switch title{
            case "Oferta":
                NotificationCenter.default.post(name: NSNotification.Name(rawValue: Constants.NotificationKeys.WorkerOffered), object: nil, userInfo: remoteMessage.appData)
                break
            case "Trabajo":
                let sender_token = remoteMessage.appData["sender_token"] as! String
                let categoria = remoteMessage.appData["categoria"] as! String
                let preciomax = remoteMessage.appData["precio_max"] as! String
                let userid = remoteMessage.appData["user_id"] as! String
                let clientName = remoteMessage.appData["user_name"] as! String
                let clientSurname = remoteMessage.appData["user_surname"] as! String
                let descripcion = remoteMessage.appData["descripcion"] as! String
                let latitude = Double(remoteMessage.appData["latitude"] as! String)
                let longitude = Double(remoteMessage.appData["longitude"] as! String)
                let currentLocation: CLLocationCoordinate2D = CLLocationCoordinate2D(latitude: latitude!, longitude: longitude!)
                let trabajo : Trabajo = Trabajo(objectId: "1", sender_token: sender_token, clienteId: userid, clientName: clientName, clientSurname: clientSurname, precioMax: Double(preciomax)!, oficio: Oficio(nombre: categoria, selected: false), descripcion: descripcion, currentLocation: currentLocation)
                if(Global.user != nil && userid != Global.user?.objectId){
                    let alert = UIAlertController(title: "Oferta de Trabajo", message:"Has recibido una oferta de trabajo", preferredStyle: UIAlertControllerStyle.alert)
                
                // add an action (button)
                let okAction = UIAlertAction(title: "Ver Oferta", style: UIAlertActionStyle.default) {
                    UIAlertAction in
                    NSLog("Ver Oferta")
                    
                    let storyboard = UIStoryboard(name: "Main", bundle: nil)
                    let viewController: UINavigationController = storyboard.instantiateViewController(withIdentifier: "TrabajoNavViewController") as! UINavigationController
                    
                    // Then push that view controller onto the navigation stack
                    
                    if var topController = self.window?.rootViewController {
                        while let presentedViewController = topController.presentedViewController {
                            topController = presentedViewController
                        }
                        let trabajoVC = viewController.viewControllers.first as! TrabajoViewController
                        
                        trabajoVC.trabajo = trabajo
                        topController.present(viewController, animated: true)
                        // topController should now be your topmost view controller
                    }
                   // let rootViewController = .rootViewController as! UINavigationController
                    
                   // rootViewController.present(viewController, animated: true)
                    
                }
                let cancelAction = UIAlertAction(title: "Rechazar", style: UIAlertActionStyle.cancel) {
                    UIAlertAction in
                    NSLog("Cancel Pressed")
                }
                    alert.addAction(okAction)
                    alert.addAction(cancelAction)
                
                // show alert
                    let alertWindow = UIWindow(frame: UIScreen.main.bounds)
                    alertWindow.rootViewController = UIViewController()
                    alertWindow.windowLevel = UIWindowLevelAlert + 1;
                    alertWindow.makeKeyAndVisible()
                    alertWindow.rootViewController?.present(alert, animated: true, completion: nil)
                }
                break
            default:
                break
        }
        
    }
    
}

