//
//  BuscadorViewController.swift
//  WorkNow
//
//  Created by Javier Arguello on 8/5/17.
//  Copyright © 2017 uoc. All rights reserved.
//

import UIKit
import MapKit
import Firebase
import FirebaseDatabase

class BuscadorViewController: UIViewController, MKMapViewDelegate, CLLocationManagerDelegate, ProfTableViewControllerDelegate{

    let dbRef:FIRDatabaseReference = FIRDatabase.database().reference(withPath: "users")
    @IBOutlet weak var profesionButton: UIButton!
    
    @IBOutlet weak var stepper: UIStepper!
    @IBOutlet weak var distanciaField: UITextField!
    let locationManager = CLLocationManager()
    var isUpdatingLocation = false
    var lastLocationError: NSError?
    let authStatus = CLLocationManager.authorizationStatus()
    var location: CLLocation?
    var pinLocation = CLLocationCoordinate2D(latitude: 51.51, longitude: -0.15)
    var firstOffer : Bool = true
    
    var totalKms : Int = 1
    
    @IBOutlet weak var categoriaOutlet: UIButton!
    @IBOutlet weak var mapView: MKMapView!
    
    @IBOutlet weak var precioTF: UITextField!
    @IBOutlet weak var descripcionTV: UITextView!
    
    func postNotification(){
        let categoria: String = (categoriaOutlet.titleLabel?.text)!
        let precio : String = precioTF.text!
        let userid : String = (Global.user?.objectId)!
        let username : String = (Global.user?.nombre)!
        let surname : String = (Global.user?.apellidos)!
        let description : String = descripcionTV.text
        let json = [
            "to": "/topics/\(Global.convertString(str: categoria))",
         "notification": [
            "body": "Has recibido una nueva oferta de trabajo",
            "title": "Trabajo"
         ],
         "data": [
                "sender_token": Global.deviceToken,
                "categoria": categoria,
                "precio_max": precio,
                "user_id": userid,
                "user_name": username,
                "user_surname": surname,
                "latitude": "\((location?.coordinate.latitude)!)",
                "longitude": "\((location?.coordinate.longitude)!)",
                "descripcion": description
            ]
         
         ] as [String: Any]
        
        var request = URLRequest(url: URL(string: "https://fcm.googleapis.com/fcm/send")!)
        request.httpMethod = "POST"
        request.addValue("application/json", forHTTPHeaderField: "Content-Type")
        request.addValue("key=\(Global.apiKey)", forHTTPHeaderField: "Authorization")
        request.httpBody = try! JSONSerialization.data(withJSONObject: json, options: [])
        
        URLSession.shared.dataTask(with:request, completionHandler: {(data, response, error) in
            if error != nil {
                print(error)
            } else {
                do {
                    guard let json = try? JSONSerialization.jsonObject(with: data!, options: .allowFragments) as? [String: Any] else { return }
                    
                    guard let errors = json?["errors"] as? [[String: Any]] else { return }
                    if errors.count > 0 {
                        // show error
                        return
                    } else {
                        // show confirmation
                    }
                }
            }
        }).resume()
        
    }
    
    
    @IBAction func buscarClick(_ sender: Any) {
        //SEND NOTIFICATION TO FIREBASE
        let precio = Double(precioTF.text!)
        let categoria: String = (categoriaOutlet.titleLabel?.text)!
        if precio == nil{
            showAlert(title: "Precio incorrecto", message: "El precio indicado no tiene un formato correcto")
        }else if(categoria == "..."){
            showAlert(title: "Categoría Incorrecta", message: "Seleccione una categoría")
        }else{
            removeAllPinsButUserLocation()
            firstOffer = true
            postNotification()
        }
        // CLEAR ALL ANNOTATION POINTS
    
    }
    
    
    
    func removeAllPinsButUserLocation()
    {
        let annotationsToRemove = mapView.annotations.filter { $0 !== mapView.userLocation }
        mapView.removeAnnotations( annotationsToRemove )
    }
    
    func getUserFromId(userId:String, completion: @escaping (_ user: User) -> ()){
        dbRef.child(userId).observe(.value, with: { snapshot in
            let dictUser = snapshot.value as! NSDictionary
            let userReturned = self.getUser(dictUser: dictUser, userId: userId)
            completion(userReturned)
        }){ (error) in
            print(error.localizedDescription)
            
        }
        
    }
    
    func showAlert(title: String,message: String) {
        let alertController = UIAlertController(title: title, message:
            message, preferredStyle: UIAlertControllerStyle.alert)
        alertController.addAction(UIAlertAction(title: "Aceptar", style: UIAlertActionStyle.default,handler: nil))
        self.present(alertController, animated: true, completion: nil)
    }
    
    
    func getUser(dictUser: NSDictionary, userId: String) -> User{
        let nombre = dictUser["nombre"] as! String
        let email = dictUser["email"] as! String
        let apellidos = dictUser["apellidos"] as! String
        let password = dictUser["password"] as! String
        let latitude = Double(dictUser["latitude"] as! String)
        let longitude = Double(dictUser["longitude"] as! String)
        var user = User(objectId: userId, nombre: nombre, apellidos: apellidos, email: email, password: password)
        user.currentLocation = CLLocationCoordinate2D(latitude: latitude!, longitude: longitude!)
        let oficios = dictUser["oficios"] as! NSArray
        var oficiosList: [Oficio] = []
        for oficio in oficios{
            let profesion = oficio as! NSDictionary
            let pNombre = profesion["nombre"] as! String
            let pSelected = profesion["selected"] as! Bool
            oficiosList += [Oficio(nombre: pNombre,selected: pSelected)]
        }
        user.oficios = oficiosList
        return user
    }
    
    func dismissKeyboard() {
        //Causes the view (or one of its embedded text fields) to resign the first responder status.
        view.endEditing(true)
    }
    
    override func viewDidLoad() {
        super.viewDidLoad()
        let tap: UITapGestureRecognizer = UITapGestureRecognizer(target: self, action: "dismissKeyboard")
        view.addGestureRecognizer(tap)
        // Do any additional setup after loading the view.
        distanciaField.text = "\(totalKms)"
        stepper.value = Double(totalKms)
        distanciaField.isEnabled = false
        self.mapView.delegate = self
        if authStatus == .notDetermined {
            locationManager.requestWhenInUseAuthorization()
            return
        }
        if authStatus == .denied || authStatus == .restricted {
            let alert = UIAlertController(title: "Location Services Disabled", message: "Please enable location services for this app in Settings.", preferredStyle: .alert)
            let okAction = UIAlertAction(title: "OK", style: .default, handler: nil)
            alert.addAction(okAction)
            present(alert, animated: true, completion: nil)
            return
        }
        
        NotificationCenter.default.addObserver(self, selector: #selector(self.notificationReceived(_:)), name: NSNotification.Name(rawValue: Constants.NotificationKeys.WorkerOffered), object: nil)
        
        startLocationManager()
    }
    
    
    
    func notificationReceived(_ notification: Notification) {
        //getting some data from userInfo is optional
        //your code here

        if(firstOffer){
            showAlert(title: "Ofertas recibidas", message: "¡Enhorabuena! Ha recibido nuevas ofertas")
            firstOffer = false
        }
        let trabajadorId: String = notification.userInfo?["trabajador_id"] as! String
        var oferta: Oferta?
        getUserFromId(userId: trabajadorId){ user in
            //let objectId : String = notification.userInfo?["oferta_id"] as! String
            let clienteId: String = notification.userInfo?["cliente_id"] as! String
            let precio: Double = Double(notification.userInfo?["precio"] as! String)!
            let nOficio: String = notification.userInfo?["oficio"] as! String
            let descripcion: String = notification.userInfo?["descripcion"] as! String
            let trabajador: User = user
            self.getUserFromId(userId: clienteId){ user in
                let cliente: User = user
                oferta = Oferta(objectId: "1", trabajador: trabajador, cliente: cliente, precio: precio, oficio: Oficio(nombre: nOficio,selected: true), descripcion: descripcion)
                
                print(oferta)
                let annotationPoint = PinAnnotation()
                annotationPoint.title = "Oferta de \(trabajador.nombre)"
                annotationPoint.subtitle = "Precio: \((oferta?.precio)!)"
                
                annotationPoint.setCoordinate(trabajador.currentLocation)
                annotationPoint.number = 1;
                self.mapView.addAnnotation(annotationPoint)
                
            }
        }
        

    }

    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
        // Dispose of any resources that can be recreated.
    }
    
    
    @IBAction func distChanged(_ sender: Any) {
        distanciaField.text = "\(Int(stepper.value))"
        totalKms = Int(stepper.value)
        centerMapOnLocation(kilometers: Int(stepper.value))
    }

    @IBAction func profesionClick(_ sender: Any) {
        performSegue(withIdentifier: "ProfesionesSegue", sender: sender)
    }
    
    func refreshProfesion(profesion: String) {
        profesionButton.setTitle(profesion, for: .normal)
    }
    @IBAction func buscarProfesionales(_ sender: Any) {
        
        
    }
    
    override func prepare(for segue: UIStoryboardSegue, sender: Any?) {
        if segue.identifier == "ProfesionesSegue" {
            let proftableViewController = segue.destination as! ProfTableViewController
            proftableViewController.delegate = self
        }
    }
    
    func startLocationManager() {
        if CLLocationManager.locationServicesEnabled() {
            locationManager.delegate = self
            locationManager.desiredAccuracy = kCLLocationAccuracyNearestTenMeters
            locationManager.startUpdatingLocation()
            isUpdatingLocation = true
        }
    }
    
    func stopLocationManager() {
        if isUpdatingLocation {
            locationManager.stopUpdatingLocation()
            locationManager.delegate = nil
            isUpdatingLocation = false
        }
    }
    
    func locationManager(_ manager: CLLocationManager, didUpdateLocations locations: [CLLocation]){
        let newLocation = locations.last!
        print("didUpdateLocations \(newLocation)")
        if newLocation.timestamp.timeIntervalSinceNow < -5 {
            return
        }
        if newLocation.horizontalAccuracy < 0 {
            return
        }
        if location == nil || location!.horizontalAccuracy > newLocation.horizontalAccuracy {
            lastLocationError = nil
            location = newLocation
            mapView.showsUserLocation = true;
            centerMapOnLocation(kilometers:totalKms)
        }
        if newLocation.horizontalAccuracy <= locationManager.desiredAccuracy {
            print("Got the desired accuracy")
            stopLocationManager();
        }
    }
    func centerMapOnLocation(kilometers: Int) {
        let regionRadius: CLLocationDistance = 1000 * Double(kilometers);
        let coordinateRegion = MKCoordinateRegionMakeWithDistance(location!.coordinate,
                                                                  regionRadius * 2.0, regionRadius * 2.0)
        mapView.setRegion(coordinateRegion, animated: true)
    }
    
    
    func locationManager(_ manager: CLLocationManager, didFailWithError error: Error) {
        print("didFailWithError \(error)")
        if error._code == CLError.Code.locationUnknown.rawValue {
            return
        }
        lastLocationError = error as NSError
        stopLocationManager()
    }

    
    func mapView(_ mapView: MKMapView, viewFor annotation: MKAnnotation) -> MKAnnotationView? {
        if annotation is PinAnnotation {
            let pinAnnotationView = MKPinAnnotationView(annotation: annotation, reuseIdentifier: "myPin")
            pinAnnotationView.isDraggable = false
            pinAnnotationView.canShowCallout = true
            pinAnnotationView.animatesDrop = true
            
            let goButton = UIButton(type: UIButtonType.custom)
            goButton.frame.size.width = 40
            goButton.frame.size.height = 40
            goButton.setImage(UIImage(named: "arrow"), for: UIControlState())
            
            pinAnnotationView.leftCalloutAccessoryView = goButton
            
            return pinAnnotationView
        }
        
        return nil
    }
    
    func mapView(_ mapView: MKMapView, annotationView view: MKAnnotationView, didChange newState: MKAnnotationViewDragState, fromOldState oldState: MKAnnotationViewDragState) {
        switch (newState) {
        case .ending, .canceling:
            
            if view.annotation is PinAnnotation {
                let coords = view.annotation?.coordinate;
                pinLocation = coords!;
                print("Latitude: \(coords?.latitude) Longitude: \(coords?.longitude)")
            }
            view.dragState = .none
        default: break
        }
    }
    func mapView(_ mapView: MKMapView, annotationView view: MKAnnotationView, calloutAccessoryControlTapped control: UIControl) {
        if let annotation = view.annotation as? PinAnnotation {
        //    currentSite = Global.sitios[annotation.number!]
            self.performSegue(withIdentifier: "SegueSitioGlobal", sender: self)
            
        }
    }

    
    /*
    // MARK: - Navigation

    // In a storyboard-based application, you will often want to do a little preparation before navigation
    override func prepare(for segue: UIStoryboardSegue, sender: Any?) {
        // Get the new view controller using segue.destinationViewController.
        // Pass the selected object to the new view controller.
    }
    */

}
