//
//  MenuViewController.swift
//  WorkNow
//
//  Created by Javier Arguello on 29/4/17.
//  Copyright © 2017 uoc. All rights reserved.
//

import UIKit
import MapKit
import FirebaseAuth
import FirebaseDatabase
import FirebaseMessaging

class MenuViewController: UITableViewController, CLLocationManagerDelegate {

    let ref:FIRDatabaseReference = FIRDatabase.database().reference(withPath: "users")
    let locationManager = CLLocationManager()
    var isUpdatingLocation = false
    var lastLocationError: NSError?
    let authStatus = CLLocationManager.authorizationStatus()
    var location: CLLocation?
    var oficiosList: [Oficio] = []
    

    func subscribeToOficios(){
        for oficio in (Global.user?.oficios)!{
            if(oficio.isSelected){
                FIRMessaging.messaging().subscribe(toTopic: Global.convertString(str: oficio.nombre))
            }
        }
    }
    
    override func viewDidLoad() {
        super.viewDidLoad()
        let userID = FIRAuth.auth()?.currentUser?.uid
        if(Global.user == nil){
        ref.child(userID!).observeSingleEvent(of: .value, with: { (snapshot) in
            // Get user value
            let value = snapshot.value as? NSDictionary
            let nombre = value?["nombre"] as! String
            let apellidos = value?["apellidos"] as! String
            let email = value?["email"] as! String
            let password = value?["password"] as! String
            Global.user = User(objectId: userID!, nombre: nombre, apellidos: apellidos, email: email, password: password)
            if let loc = self.location{
                Global.user?.currentLocation = loc.coordinate
            }
            let oficios = value?["oficios"] as! NSArray
            for of in oficios{
                let obj = of as! NSDictionary
                let nombre = obj["nombre"] as! String
                let selected = obj["selected"] as! Bool
                Global.user?.oficios+=[Oficio(nombre: nombre,selected: selected)]
            }
            self.subscribeToOficios()
            // ...
        }) { (error) in
            print(error.localizedDescription)
            }
        }
        startLocationManager()
        // Do any additional setup after loading the view.
    }
    
    
    func getOficiosDB(ofRef: FIRDatabaseReference) {
        
        ofRef.observeSingleEvent(of: .value, with: { (snapshot) in
            // Get user value
            let value = snapshot.value as? NSArray
            for v in value!{
                let obj = v as! NSDictionary
                let nombre = obj["nombre"] as! String
                let selected = obj["selected"] as! Bool
                self.oficiosList+=[Oficio(nombre: nombre,selected: selected)]
            }
            // ...
        }) { (error) in
            print(error.localizedDescription)
        }
        
        
    }

    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
        // Dispose of any resources that can be recreated.
    }
    
    @IBAction func logoutClick(_ sender: Any) {
        Global.user = nil
        performSegue(withIdentifier: "LogoutSegue", sender: sender)
    }

    func startLocationManager() {
        if CLLocationManager.locationServicesEnabled() {
            locationManager.delegate = self
            locationManager.desiredAccuracy = kCLLocationAccuracyNearestTenMeters
            locationManager.startUpdatingLocation()
            isUpdatingLocation = true
        }
    }
    
    func stopLocationManager() {
        if isUpdatingLocation {
            locationManager.stopUpdatingLocation()
            locationManager.delegate = nil
            isUpdatingLocation = false
        }
    }
    
    func locationManager(_ manager: CLLocationManager, didUpdateLocations locations: [CLLocation]){
        let newLocation = locations.last!
        print("didUpdateLocations \(newLocation)")
        if newLocation.timestamp.timeIntervalSinceNow < -5 {
            return
        }
        if newLocation.horizontalAccuracy < 0 {
            return
        }
        if location == nil || location!.horizontalAccuracy > newLocation.horizontalAccuracy {
            lastLocationError = nil
            location = newLocation
            Global.user?.currentLocation = (location?.coordinate)!
            updateLocationInDB()
        }
        if newLocation.horizontalAccuracy <= locationManager.desiredAccuracy {
            print("Got the desired accuracy")
            stopLocationManager();
        }
    }
    
    
    func locationManager(_ manager: CLLocationManager, didFailWithError error: Error) {
        print("didFailWithError \(error)")
        if error._code == CLError.Code.locationUnknown.rawValue {
            return
        }
        lastLocationError = error as NSError
        stopLocationManager()
    }
    
    func updateLocationInDB() {
        let userID = FIRAuth.auth()?.currentUser?.uid
            let child = ref.child(userID!)
            let childUpdates = ["longitude": "\(location!.coordinate.longitude)",
                                "latitude": "\(location!.coordinate.latitude)"]
            child.updateChildValues(childUpdates)
        
    }
    
    override func prepare(for segue: UIStoryboardSegue, sender: Any?) {
        if segue.identifier == "GoToCategorias" {
            let catViewController = segue.destination as! CategoriasVController
            let ofRef:FIRDatabaseReference = FIRDatabase.database().reference(withPath: "oficios")
            ofRef.observeSingleEvent(of: .value, with: { (snapshot) in
                // Get user value
                let value = snapshot.value as? NSArray
                for v in value!{
                    let obj = v as! NSDictionary
                    let nombre = obj["nombre"] as! String
                    let selected = obj["selected"] as! Bool
                    self.oficiosList+=[Oficio(nombre: nombre,selected: selected)]
                }
                catViewController.profesionesList = self.oficiosList
                // ...
            }) { (error) in
                print(error.localizedDescription)
            }
        }
    }
    
    /*
    // MARK: - Navigation

    // In a storyboard-based application, you will often want to do a little preparation before navigation
    override func prepare(for segue: UIStoryboardSegue, sender: Any?) {
        // Get the new view controller using segue.destinationViewController.
        // Pass the selected object to the new view controller.
    }
    */

}
