//
//  RegistroViewController.swift
//  WorkNow
//
//  Created by Javier Arguello on 21/4/17.
//  Copyright © 2017 uoc. All rights reserved.
//

import UIKit
import Firebase
import FirebaseAuth
class RegistroViewController: UIViewController, UIPopoverPresentationControllerDelegate, PopOverViewControllerDelegate {

    let ref:FIRDatabaseReference = FIRDatabase.database().reference(withPath: "users")
    let ofRef:FIRDatabaseReference = FIRDatabase.database().reference(withPath: "oficios")
    @IBOutlet weak var nombreTF: UITextField!
    @IBOutlet weak var apellidosTF: UITextField!
    @IBOutlet weak var emailTF: UITextField!
    @IBOutlet weak var passTF: UITextField!
    @IBOutlet weak var pass2TF: UITextField!
    
    var oficiosList: [Oficio] = []
    var popoverViewController: PopOverViewController!
    override func viewDidLoad() {
        let tap: UITapGestureRecognizer = UITapGestureRecognizer(target: self, action: "dismissKeyboard")
        view.addGestureRecognizer(tap)
        super.viewDidLoad()
        getOficiosDB()
        // Do any additional setup after loading the view.
    }
    
    func dismissKeyboard() {
        //Causes the view (or one of its embedded text fields) to resign the first responder status.
        view.endEditing(true)
    }
    
    @IBAction func facebookClick(_ sender: Any) {
        showAlert(title: "Facebook no disponible",message: "La versión actual de la aplicación no dispone de registro a través de Facebook. En breves será incluído en próximas versiones. Disculpe las molestias.")
    }
    @IBAction func googleClick(_ sender: Any) {
        showAlert(title: "Google no disponible",message: "La versión actual de la aplicación no dispone de registro a través de Google. En breves será incluído en próximas versiones. Disculpe las molestias.")
    }
    func getOficiosDB() {
        
        ofRef.observeSingleEvent(of: .value, with: { (snapshot) in
            let value = snapshot.value as? NSArray
            self.oficiosList = []
            for v in value!{
                let obj = v as! NSDictionary
                let nombre = obj["nombre"] as! String
                let selected = obj["selected"] as! Bool
                self.oficiosList+=[Oficio(nombre: nombre,selected: selected)]
            }
        }) { (error) in
            print(error.localizedDescription)
        }
    
    
    }
    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
        // Dispose of any resources that can be recreated.
    }
    
    func showAlert(title: String,message: String) {
        let alertController = UIAlertController(title: title, message:
            message, preferredStyle: UIAlertControllerStyle.alert)
        alertController.addAction(UIAlertAction(title: "Aceptar", style: UIAlertActionStyle.default,handler: nil))
        self.present(alertController, animated: true, completion: nil)
    }
    
    func checkEmail(email:String)-> Bool{
        let emailRegEx = "[A-Z0-9a-z._%+-]+@[A-Za-z0-9.-]+\\.[A-Za-z]{2,}"
        
        let emailTest = NSPredicate(format:"SELF MATCHES %@", emailRegEx)
        return emailTest.evaluate(with: email)
    }
    
    func fieldsEmpty() -> Bool{
        return nombreTF.text == nil || nombreTF.text?.characters.count == 0 || apellidosTF.text == nil || apellidosTF.text?.characters.count == 0
    }

    @IBAction func registroClick(_ sender: Any) {
        let pass: String = passTF.text!
        let email: String = emailTF.text!
        let pass2: String = pass2TF.text!
        if(!checkEmail(email: email)){
            showAlert(title: "Email incorrecto",message: "Introduzca un email correcto")
        }else if(pass.characters.count < 8 ){
            showAlert(title: "Password incorrecto",message: "El password debe tener al menos 8 caracteres")
        }else if(pass != pass2){
            showAlert(title: "Password incorrecto",message: "Los passwords introducidos no coinciden en ambos campos.")
        }else if(fieldsEmpty()){
            showAlert(title: "Campos vacios",message: "No están permitidos campos vacíos.")
        }else{
            FIRAuth.auth()?.createUser(withEmail: email, password: pass, completion: {(user, error) in
                
                if error != nil {
                    self.showAlert(title: "Registro fallido",message: error!.localizedDescription)
                    return
                }else{

                    var usuario:User = User(objectId: "1",nombre: self.nombreTF.text!,apellidos: self.apellidosTF.text!,email: email,password: pass)
                    let userRef = self.ref.child(user!.uid)
                    
                    usuario.oficios = self.oficiosList
                    userRef.setValue(usuario.toAnyObject())
                    userRef.child("oficios").setValue(usuario.oficiosToAnyObjectList())
                    
                    
                    let alertController = UIAlertController(title: "Registro correcto", message:
                        "Usuario registrado correctamente", preferredStyle: UIAlertControllerStyle.alert)
                    alertController.addAction(UIAlertAction(title: "Aceptar", style: UIAlertActionStyle.default,handler: self.goToLogin))
                    self.present(alertController, animated: true, completion: nil)
                    return 
                }
            })
        }
        
    }
    /*
    // MARK: - Navigation

    // In a storyboard-based application, you will often want to do a little preparation before navigation
    override func prepare(for segue: UIStoryboardSegue, sender: Any?) {
        // Get the new view controller using segue.destinationViewController.
        // Pass the selected object to the new view controller.
    }
    */
    
    func goToLogin(action: UIAlertAction){
        if let navController = self.navigationController {
            navController.popViewController(animated: true)
        }
    }
    @IBAction func officiosClick(_ sender: Any) {
        
        
    }
    
    func updateOficios(oficiosList: [Oficio]) {
        self.oficiosList = oficiosList
    }
    
    override func prepare(for segue: UIStoryboardSegue, sender: Any?) {
        if segue.identifier == "PopOverSegue" {
            popoverViewController = segue.destination as! PopOverViewController
            popoverViewController.modalPresentationStyle = UIModalPresentationStyle.popover
            popoverViewController.popoverPresentationController!.delegate = self
            popoverViewController.delegate = self
            popoverViewController.oficiosList = self.oficiosList
        }
    }
    func adaptivePresentationStyle(for controller: UIPresentationController) -> UIModalPresentationStyle {
        return UIModalPresentationStyle.none
    }
    
    func adaptivePresentationStyle(for controller: UIPresentationController, traitCollection: UITraitCollection) -> UIModalPresentationStyle {
        return UIModalPresentationStyle.none
    }
}
