//
//  TrabajoViewController.swift
//  WorkNow
//
//  Created by Javier Arguello on 12/6/17.
//  Copyright © 2017 uoc. All rights reserved.
//

import UIKit
import MapKit

class TrabajoViewController: UIViewController, MKMapViewDelegate, CLLocationManagerDelegate {


    @IBOutlet weak var ofertaTF: UITextField!
    @IBOutlet weak var comentariosTF: UITextField!
    @IBOutlet weak var mapView: MKMapView!
    @IBOutlet weak var descrTF: UITextView!
    @IBOutlet weak var precioTF: UITextField!
    
    @IBOutlet weak var usuarioLabel: UILabel!
    let locationManager = CLLocationManager()
    var isUpdatingLocation = false
    var lastLocationError: NSError?
    let authStatus = CLLocationManager.authorizationStatus()
    var location: CLLocation?
    var pinLocation = CLLocationCoordinate2D(latitude: 51.51, longitude: -0.15)
    var trabajo: Trabajo?
    override func viewDidLoad() {
        let tap: UITapGestureRecognizer = UITapGestureRecognizer(target: self, action: "dismissKeyboard")
        view.addGestureRecognizer(tap)
        super.viewDidLoad()
        if trabajo != nil {
            let pinLocation : CLLocation = CLLocation(latitude: (trabajo?.currentLocation.latitude)!, longitude: (trabajo?.currentLocation.longitude)!)
            centerMapOnLocation(pinlocation: pinLocation, kilometers: 10)
            usuarioLabel.text = "\((trabajo?.clientName)!) \((trabajo?.clientSurname)!)"
            descrTF.text = trabajo?.descripcion
            descrTF.isEditable = false
            let annotationPoint = PinAnnotation()
            annotationPoint.title = "Oferta de Trabajo"
            //annotationPoint.subtitle = "Valoracion: "
            
            annotationPoint.setCoordinate(pinLocation.coordinate)
            annotationPoint.number = 1;
            self.mapView.addAnnotation(annotationPoint)
        }
        self.mapView.delegate = self
        if authStatus == .notDetermined {
            locationManager.requestWhenInUseAuthorization()
            return
        }
        if authStatus == .denied || authStatus == .restricted {
            let alert = UIAlertController(title: "Location Services Disabled", message: "Please enable location services for this app in Settings.", preferredStyle: .alert)
            let okAction = UIAlertAction(title: "OK", style: .default, handler: nil)
            alert.addAction(okAction)
            present(alert, animated: true, completion: nil)
            return
        }
        // Do any additional setup after loading the view.
    }
    
    func showAlert(title: String,message: String) {
        let alertController = UIAlertController(title: title, message:
            message, preferredStyle: UIAlertControllerStyle.alert)
        alertController.addAction(UIAlertAction(title: "Aceptar", style: UIAlertActionStyle.default,handler: nil))
        self.present(alertController, animated: true, completion: nil)
    }
    
    func postNotification(){
       
        let precio : String = precioTF.text!
        let json = [
            "to": trabajo?.sender_token,
            "notification": [
                "body": "Has recibido una nueva oferta",
                "title": "Oferta"
            ],
            "data": [
                "sender_token": Global.deviceToken,
                "trabajador_id": Global.user?.objectId,
                "cliente_id": trabajo?.clienteId,
                "precio": precio,
                "oficio": "Mecánica",
                "descripcion": comentariosTF.text!
            ]
            
            ] as [String: Any]
        
        var request = URLRequest(url: URL(string: "https://fcm.googleapis.com/fcm/send")!)
        request.httpMethod = "POST"
        request.addValue("application/json", forHTTPHeaderField: "Content-Type")
        request.addValue("key=\(Global.apiKey)", forHTTPHeaderField: "Authorization")
        request.httpBody = try! JSONSerialization.data(withJSONObject: json, options: [])
        
        URLSession.shared.dataTask(with:request, completionHandler: {(data, response, error) in
            if error != nil {
                print(error)
            } else {
                do {
                    guard let json = try? JSONSerialization.jsonObject(with: data!, options: .allowFragments) as? [String: Any] else { return }
                    
                    guard let errors = json?["errors"] as? [[String: Any]] else { return }
                    if errors.count > 0 {
                        // show error
                        return
                    } else {
                        // show confirmation
                    }
                }
            }
        }).resume()
        
    }

    
    @IBAction func enviarOferta(_ sender: Any) {
        let precio = Double(precioTF.text!)
        if precio == nil{
            showAlert(title: "Precio incorrecto", message: "El precio indicado no tiene un formato correcto")
        }else{
            postNotification()
        }
    }
    
    func dismissKeyboard() {
        //Causes the view (or one of its embedded text fields) to resign the first responder status.
        view.endEditing(true)
    }

    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
        // Dispose of any resources that can be recreated.
    }
    
    func locationManager(_ manager: CLLocationManager, didUpdateLocations locations: [CLLocation]){
        let newLocation = locations.last!
        print("didUpdateLocations \(newLocation)")
        if newLocation.timestamp.timeIntervalSinceNow < -5 {
            return
        }
        if newLocation.horizontalAccuracy < 0 {
            return
        }
        if location == nil || location!.horizontalAccuracy > newLocation.horizontalAccuracy {
            lastLocationError = nil
            location = newLocation
            mapView.showsUserLocation = true
            
        }
        if newLocation.horizontalAccuracy <= locationManager.desiredAccuracy {
            
            
            print("Got the desired accuracy")
            stopLocationManager();
        }
    }
    func centerMapOnLocation(pinlocation: CLLocation,kilometers: Int) {
        let regionRadius: CLLocationDistance = 1000 * Double(kilometers);
        let coordinateRegion = MKCoordinateRegionMakeWithDistance(pinLocation,
                                                                  regionRadius * 2.0, regionRadius * 2.0)
        mapView.setRegion(coordinateRegion, animated: true)
    }
    
    
    func locationManager(_ manager: CLLocationManager, didFailWithError error: Error) {
        print("didFailWithError \(error)")
        if error._code == CLError.Code.locationUnknown.rawValue {
            return
        }
        lastLocationError = error as NSError
        stopLocationManager()
    }

    func stopLocationManager() {
        if isUpdatingLocation {
            locationManager.stopUpdatingLocation()
            locationManager.delegate = nil
            isUpdatingLocation = false
        }
    }
    func mapView(_ mapView: MKMapView, viewFor annotation: MKAnnotation) -> MKAnnotationView? {
        if annotation is PinAnnotation {
            let pinAnnotationView = MKPinAnnotationView(annotation: annotation, reuseIdentifier: "myPin")
            pinAnnotationView.isDraggable = false

            return pinAnnotationView
        }
        
        return nil
    }
    
    func mapView(_ mapView: MKMapView, annotationView view: MKAnnotationView, didChange newState: MKAnnotationViewDragState, fromOldState oldState: MKAnnotationViewDragState) {
        switch (newState) {
        case .ending, .canceling:
            
            view.dragState = .none
        default: break
        }
    }
    func mapView(_ mapView: MKMapView, annotationView view: MKAnnotationView, calloutAccessoryControlTapped control: UIControl) {
    }
    /*
    // MARK: - Navigation

    // In a storyboard-based application, you will often want to do a little preparation before navigation
    override func prepare(for segue: UIStoryboardSegue, sender: Any?) {
        // Get the new view controller using segue.destinationViewController.
        // Pass the selected object to the new view controller.
    }
    */

}
