//
//  ViewController.swift
//  WorkNow
//
//  Created by Javier Arguello on 21/4/17.
//  Copyright © 2017 uoc. All rights reserved.
//

import UIKit
import Firebase
import FirebaseAuth

class ViewController: UIViewController, UITextFieldDelegate {

    
    @IBOutlet weak var emailField: UITextField!
    @IBOutlet weak var passField: UITextField!
    
    override func viewWillAppear(_ animated: Bool) {
        self.navigationController?.isNavigationBarHidden = true
    }
    
    override func viewWillDisappear(_ animated: Bool) {
        self.navigationController?.isNavigationBarHidden = false
    }
    override func viewDidLoad() {
        let tap: UITapGestureRecognizer = UITapGestureRecognizer(target: self, action: "dismissKeyboard")
        view.addGestureRecognizer(tap)
        super.viewDidLoad()
        self.emailField.delegate = self
        self.passField.delegate = self
       /* NotificationCenter.default.addObserver(self, selector: #selector(keyboardWillShow), name: NSNotification.Name.UIKeyboardWillShow, object: nil)
        NotificationCenter.default.addObserver(self, selector: #selector(keyboardWillHide), name: NSNotification.Name.UIKeyboardWillHide, object: nil)*/

        // Do any additional setup after loading the view, typically from a nib.
    }

    func dismissKeyboard() {
        //Causes the view (or one of its embedded text fields) to resign the first responder status.
        view.endEditing(true)
    }
    
    func keyboardWillShow(notification: NSNotification) {
        if let keyboardSize = (notification.userInfo?[UIKeyboardFrameBeginUserInfoKey] as? NSValue)?.cgRectValue {
            if self.view.frame.origin.y == 0{
                self.view.frame.origin.y -= keyboardSize.height
            }
        }
    }
    
    func keyboardWillHide(notification: NSNotification) {
        if let keyboardSize = (notification.userInfo?[UIKeyboardFrameBeginUserInfoKey] as? NSValue)?.cgRectValue {
            if self.view.frame.origin.y != 0{
                self.view.frame.origin.y += keyboardSize.height
            }
        }
    }
    
    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
        // Dispose of any resources that can be recreated.
    }

    @IBAction func facebookClick(_ sender: Any) {
        showAlert(title: "Facebook no disponible",message: "La versión actual de la aplicación no dispone de logado a través de Facebook. En breves será incluído en próximas versiones. Disculpe las molestias.")
    }
    @IBAction func googleClick(_ sender: Any) {
        showAlert(title: "Google no disponible",message: "La versión actual de la aplicación no dispone de logado a través de Google. En breves será incluído en próximas versiones. Disculpe las molestias.")
    }
    func loginServer(email: String, pass: String) -> Bool{
        FIRAuth.auth()?.signIn(withEmail: email, password: pass){ (user, error) in
            if error != nil{
                self.showAlert(title: "Error al iniciar sesión",message: (error?.localizedDescription)!)
            }else{
                self.performSegue(withIdentifier: "LoginToMenu", sender: self)
            }
        }
        return true
    }
    
    func checkEmail(email:String)-> Bool{
        let emailRegEx = "[A-Z0-9a-z._%+-]+@[A-Za-z0-9.-]+\\.[A-Za-z]{2,}"
        
        let emailTest = NSPredicate(format:"SELF MATCHES %@", emailRegEx)
        return emailTest.evaluate(with: email)
    }

    @IBAction func loginClick(_ sender: Any) {
        
        let email: String = emailField.text!
        let pass : String = passField.text!
        
        if(!checkEmail(email: email)){
            showAlert(title: "Email incorrecto",message: "Introduzca un email correcto")
        }else if(pass.characters.count < 8 ){
            showAlert(title: "Password incorrecto",message: "Password incorrecto")
        }else{
            loginServer(email: email, pass: pass)
        }
    }
    func showAlert(title: String,message: String) {
        let alertController = UIAlertController(title: title, message:
            message, preferredStyle: UIAlertControllerStyle.alert)
        alertController.addAction(UIAlertAction(title: "Aceptar", style: UIAlertActionStyle.default,handler: nil))
        
        self.present(alertController, animated: true, completion: nil)
    }
    
    func textFieldShouldReturn(_ textField: UITextField) -> Bool {
        self.view.endEditing(true)
        return false
    }
}

